#!/usr/bin/env python

"""
Updated on 2021/06/23 by pan xia
"""

# Edit this script to add your team's training code. Some functions are *required*,
# but you can edit most parts of the required functions, remove non-required functions, and add your own functions.

################################################################################
#
# Imported functions and variables
#
################################################################################

# Import functions. These functions are not required. You can change or remove them.
from helper_code import *
import numpy as np, os, sys, joblib, json
from keras.losses import categorical_crossentropy
from model_prediction import load_mymodel
from sklearn.model_selection import train_test_split
from preprocess_features_labels import get_classes, dataPrepare, data_uniform, get_12ECG_features, usedClasses
from model_prediction import myPredict
from CNN_MultiScale import XpModel

import tensorflow as tf
from tensorflow.compat.v1 import InteractiveSession
config = tf.ConfigProto()
config.gpu_options.allow_growth = True
session = InteractiveSession(config=config)

# Define the Challenge lead sets. These variables are not required. You can change or remove them.
twelve_leads = ('I', 'II', 'III', 'aVR', 'aVL', 'aVF', 'V1', 'V2', 'V3', 'V4', 'V5', 'V6')
six_leads = ('I', 'II', 'III', 'aVR', 'aVL', 'aVF')
four_leads = ('I', 'II', 'III', 'V2')
three_leads = ('I', 'II', 'V2')
two_leads = ('I', 'II')
lead_sets = (twelve_leads, six_leads, four_leads, three_leads, two_leads)  #

################################################################################
#
# Training model function
#
################################################################################


# Train your model. This function is *required*. You should edit this function to add your code,
# but do *not* change the arguments of this function.
def training_code(data_directory, model_directory):

    # **************************载入数据文件**************************
    print('Loading data...')
    header_files, recording_files = find_challenge_files(data_directory)
    num_recordings = len(recording_files)

    if not num_recordings:
        raise Exception('No data was provided.')

    # Create a folder for the model if it does not already exist.
    if not os.path.isdir(model_directory):
        os.mkdir(model_directory)

    # **************************提取打分数据类别**************************
    # get 30 scored classes
    scored_classes = set()
    for header_file in header_files:
        header = load_header(header_file)
        recording_labels = get_labels(header)
        recording_scored_classes = []
        for label in recording_labels:
            if label in usedClasses:
                recording_scored_classes.append(label)
        scored_classes |= set(recording_scored_classes)

    if all(is_integer(x) for x in scored_classes):
        scored_classes = sorted(scored_classes, key=lambda x: int(x))  # Sort classes numerically if numbers.
    else:
        scored_classes = sorted(scored_classes)  # Sort classes alphanumerically if not numbers.

    for leads in lead_sets:
        # **************************加载数据、解析特征 **************************
        # Extract the features and labels from the dataset.
        totalFeatures, totalLabels = [], []
        totalFeatures, totalLabels, totalSeglengthFS = dataPrepare(header_files, recording_files, leads, scored_classes)

        # **************************数据归一化处理 **************************
        uniform_features, uniform_Labels, theMean, theStd = data_uniform(totalFeatures, totalLabels)
        print(uniform_features.shape, uniform_Labels.shape, scored_classes)

        # **************************训练模型**************************
        print('Training model...')

        epochs = 35   # 模型训练轮数
        myModel = XpModel()

        # 数据集划分
        x_train, x_test, y_train, y_test = train_test_split(uniform_features, uniform_Labels, test_size=0.01)

        # 开始训练模型
        model = myModel.startTrain(x_train, y_train, len(scored_classes), loss=categorical_crossentropy, epochs=epochs,
                                   useDataAugment=False)  # #categorical_crossentropy,'mse'

        # 保存模型和均值、标准差、class等辅助信息
        print('Saving model...')
        model_filename = get_model_filename(leads)
        model.save(os.path.join(model_directory, model_filename))

        myDict = {'theMean': theMean, 'theStd': theStd, 'class': scored_classes, 'leads': leads}
        root, _ = os.path.splitext(model_filename)
        json.dump(myDict, open(os.path.join(model_directory, root + '.json'), 'w'))
    ################################################################################


################################################################################
#
# Running trained model function
#
################################################################################


# Run your trained model. This function is *required*. You should edit this function to add your code,
# but do *not* change the arguments of this function.
def run_model(model, header, recording):
    theMean = model[0]
    theStd = model[1]
    classes = model[2]
    leads = model[3]
    model = model[4]  # 深度模型存储于此

    # Load features.
    single_recording_features, _ = get_12ECG_features(header, recording, leads)
    single_recording_features = np.asarray(single_recording_features)

    feats_uniform = (single_recording_features - theMean) / theStd
    feats_uniform[np.isnan(feats_uniform)] = 0.01
    feats_uniform[np.isinf(feats_uniform)] = 0.01
    labels, probabilities = myPredict(model, [feats_uniform])

    return classes, labels, probabilities


################################################################################
#
# File I/O functions
#
################################################################################

# Save a trained model. This function is not required. You can change or remove it.
def save_model(model_directory, leads, classes, theMean, theStd, classifier):
    d = {'leads': leads, 'classes': classes, 'theMean': theMean, 'theStd': theStd, 'classifier': classifier}
    filename = os.path.join(model_directory, get_model_filename(leads))
    joblib.dump(d, filename, protocol=0)


# Load a trained model. This function is *required*. You should edit this function to add your code,
# but do *not* change the arguments of this function.
def load_model(model_directory, leads):

    root_leads, _ = os.path.splitext(get_model_filename(leads))
    dirs = os.listdir(model_directory)
    model = []
    myDict = []
    for file in dirs:
        root_file, _ = os.path.splitext(file)
        if root_leads == root_file:
            if str(file).endswith(".h5"):
                model.append(load_mymodel(model_directory, file))  # 21比赛与内置的函数名相撞，用名为load_mymodel()的函数改写load_model()
            elif str(file).endswith(".json"):
                myDict = json.load(open(os.path.join(model_directory, file), 'r'))  # 读取模型辅助信息

    theMean = myDict['theMean']
    theStd = myDict['theStd']
    classes = myDict['class']
    leads = myDict['leads']
    return [theMean, theStd, classes, leads, model]


# Define the filename(s) for the trained models. This function is not required. You can change or remove it.
def get_model_filename(leads):
    sorted_leads = sort_leads(leads)
    return 'model_' + '-'.join(sorted_leads) + '.h5'



